@@ -474,7 +474,8 @@ class RadialGridDescriptor(ABC):
     def ceil(self, r):
         return np.ceil(self.r2g(r)).astype(int)
 
-    def spline(self, a_g, rcut=None, l=0, points=None):
+    def spline(self, a_g, rcut=None, l=0, points=None,
+               backwards_compatible=True):
         """Create spline representation of a radial function f(r).
 
         The spline represents a rescaled version of the function, f(r) / r^l.
@@ -496,17 +497,21 @@ class RadialGridDescriptor(ABC):
 
         # Interpolate to a uniform radial grid (for the spline representation)
         r_i = np.linspace(0, rcut, points + 1)
-        g_i = np.clip((self.r2g(r_i) + 0.5).astype(int), 1, N - 2)
-        r1_i = self.r_g[g_i - 1]
-        r2_i = self.r_g[g_i]
-        r3_i = self.r_g[g_i + 1]
-        x1_i = (r_i - r2_i) * (r_i - r3_i) / (r1_i - r2_i) / (r1_i - r3_i)
-        x2_i = (r_i - r1_i) * (r_i - r3_i) / (r2_i - r1_i) / (r2_i - r3_i)
-        x3_i = (r_i - r1_i) * (r_i - r2_i) / (r3_i - r1_i) / (r3_i - r2_i)
-        b1_i = b_g[g_i - 1]
-        b2_i = b_g[g_i]
-        b3_i = b_g[g_i + 1]
-        b_i = b1_i * x1_i + b2_i * x2_i + b3_i * x3_i
+        if not backwards_compatible:
+            from scipy.interpolate import CubicSpline
+            b_i = CubicSpline(self.r_g[:N], b_g, bc_type='clamped')(r_i)
+        else:
+            g_i = np.clip((self.r2g(r_i) + 0.5).astype(int), 1, N - 2)
+            r1_i = self.r_g[g_i - 1]
+            r2_i = self.r_g[g_i]
+            r3_i = self.r_g[g_i + 1]
+            x1_i = (r_i - r2_i) * (r_i - r3_i) / (r1_i - r2_i) / (r1_i - r3_i)
+            x2_i = (r_i - r1_i) * (r_i - r3_i) / (r2_i - r1_i) / (r2_i - r3_i)
+            x3_i = (r_i - r1_i) * (r_i - r2_i) / (r3_i - r1_i) / (r3_i - r2_i)
+            b1_i = b_g[g_i - 1]
+            b2_i = b_g[g_i]
+            b3_i = b_g[g_i + 1]
+            b_i = b1_i * x1_i + b2_i * x2_i + b3_i * x3_i
 
         return Spline.from_data(l, rcut, b_i)
 
