@@ -352,6 +352,9 @@ class Matrix(XP):
             assert self.dist.comm.size == slcomm.size
             H = self
 
+        if limit == H.shape[0]:
+            limit = None
+
         if limit:
             eps = self.xp.empty(limit)
         else:
@@ -834,9 +837,13 @@ class CuPyDistribution(MatrixDistribution):
                         return
                     if c.data.size > 0:
                         assert beta in [0.0, 1.0]
+                        # CuPy doesn't have dsyrk, so we roll our own:
                         cp.cublas.gemm('N', 'H',
                                        a.data, b.data, c.data,
-                                       alpha, beta)
+                                       0.5 * alpha, beta)
+                        cp.cublas.gemm('N', 'H',
+                                       b.data, a.data, c.data,
+                                       0.5 * alpha, 1.0)
             else:
                 1 / 0
                 assert opa == 'C' and opb == 'N'
