@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from functools import cached_property
 from math import pi
-from typing import Sequence, Literal
+from typing import Sequence, Literal, TYPE_CHECKING
 import numpy as np
 
 import gpaw.fftw as fftw
@@ -18,6 +18,9 @@ from gpaw.typing import (Array1D, Array2D, Array3D, Array4D, ArrayLike1D,
 from gpaw.new.c import add_to_density, add_to_density_gpu, symmetrize_ft
 from gpaw.fd_operators import Gradient
 
+if TYPE_CHECKING:
+    import plotly.graph_objects as go
+
 
 class UGDesc(Domain):
     def __init__(self,
@@ -182,7 +185,7 @@ class UGDesc(Domain):
                                 *,
                                 qspiral_v=None,
                                 atomdist=None,
-                                integral=None,
+                                integrals=None,
                                 cut=False,
                                 xp=None):
         """Create UGAtomCenteredFunctions object."""
@@ -191,7 +194,7 @@ class UGDesc(Domain):
                                        positions,
                                        self,
                                        atomdist=atomdist,
-                                       integral=integral,
+                                       integrals=integrals,
                                        cut=cut,
                                        xp=xp)
 
@@ -269,15 +272,20 @@ class UGDesc(Domain):
             return fftw.create_plans([0, 0, 0], dtype)
 
     def ranks_from_fractional_positions(self,
-                                        fracpos_ac: Array2D) -> Array1D:
-        rank_ac = np.floor(fracpos_ac * self.parsize_c).astype(int)
+                                        relpos_ac: Array2D) -> Array1D:
+        rank_ac = np.floor(relpos_ac * self.parsize_c).astype(int)
         if (rank_ac < 0).any() or (rank_ac >= self.parsize_c).any():
             raise ValueError('Positions outside cell!')
         return np.ravel_multi_index(rank_ac.T, self.parsize_c)  # type: ignore
 
-    def ecut_max(self):
-        dv_cv = self.cell_cv / self.size_c[:, np.newaxis]
-        return 0.5 * np.pi**2 / (dv_cv**2).sum(1).max()  # or min ??????
+    def ekin_max(self) -> float:
+        """Maximum value of ekin so that all 0.5 * G^2 < ekin.
+
+        In 1D, this will be 0.5*(pi/h)^2 where h is the grid-spacing.
+        """
+        # Height of reciprocal cell (squared):
+        b2_c = np.pi**2 / (self.cell_cv**2).sum(1)
+        return 0.5 * (self.size_c**2 * b2_c).min()
 
 
 class UGArray(DistributedArrays[UGDesc]):
@@ -374,6 +382,12 @@ class UGArray(DistributedArrays[UGDesc]):
         x = np.arange(grid.start_c[c], grid.end_c[c]) * dx
         return x, as_np(y)
 
+    def to_complex(self) -> UGArray:
+        """Return a copy with dtype=complex."""
+        c = self.desc.new(dtype=complex).empty()
+        c.data[:] = self.data
+        return c
+
     def scatter_from(self, data=None):
         """Scatter data from rank-0 to all ranks."""
         if isinstance(data, UGArray):
@@ -438,14 +452,27 @@ class UGArray(DistributedArrays[UGDesc]):
         return out
 
     def fft(self, plan=None, pw=None, out=None):
-        """Do FFT.
+        r"""Do FFT.
 
+        Returns:
+            PWArray with values
         :::
+                          _ _
+           _    1  / _  -iG.r   _
+         C(G) = -- |dr e      f(r),
+                V  /
 
-                _ _
-          / _  iG.r  _
-          |dr e    f(r)
-          /
+        where `C(\bG)` are the plane wave coefficients and V is the cell
+        volume.
+
+        Parameters
+        ----------
+        plan:
+            Plan for FFT.
+        pw:
+            Target PW description.
+        out:
+            Target PWArray object.
         """
         assert self.dims == ()
         if out is None:
@@ -453,6 +480,9 @@ class UGArray(DistributedArrays[UGDesc]):
             out = pw.empty(xp=self.xp)
         if pw is None:
             pw = out.desc
+        if pw.dtype != self.desc.dtype:
+            raise TypeError(
+                f'Type mismatch: {self.desc.dtype} -> {pw.dtype}')
         input = self
         if self.desc.comm.size > 1:
             input = input.gather()
@@ -776,20 +806,20 @@ class UGArray(DistributedArrays[UGDesc]):
         self.desc.comm.sum(d_v)
         return d_v
 
-    def scaled(self, s: float, v: float = 1.0):
+    def scaled(self, cell: float, values: float = 1.0) -> UGArray:
         """Create new scaled UGArray object.
 
-        Unit cell axes are multiplied by `s` and data by `v`.
+        Unit cell axes are multiplied by `cell` and data by `values`.
         """
         grid = self.desc
-        grid = UGDesc(cell=grid.cell_cv * s,
+        grid = UGDesc(cell=grid.cell_cv * cell,
                       size=grid.size_c,
                       pbc=grid.pbc_c,
                       zerobc=grid.zerobc_c,
                       kpt=(grid.kpt_c if grid.kpt_c.any() else None),
                       dtype=grid.dtype,
                       comm=grid.comm)
-        return UGArray(grid, self.dims, self.comm, self.data * v)
+        return UGArray(grid, self.dims, self.comm, self.data * values)
 
     def add_ked(self,
                 occ_n: Array1D,
@@ -809,3 +839,25 @@ class UGArray(DistributedArrays[UGDesc]):
         a = super().redist(domain, comm1, comm2)
         assert isinstance(a, UGArray)
         return a
+
+    def isosurface(self, show=True, **kwargs) -> go.Isosurface:
+        import plotly.graph_objects as go
+        values = self.data
+        assert values.ndim == 3
+        if values.dtype == complex:
+            values = abs(values)
+        x, y, z = (c.T.flatten() for c in self.desc.xyz().T)
+        vmin = values.min()
+        vmax = values.max()
+        kwargs = {
+            'isomin': vmin + (vmax - vmin) * 0.1,
+            'isomax': vmax - (vmax - vmin) * 0.1,
+            'caps': dict(x_show=False,
+                         y_show=False,
+                         z_show=False),
+            **kwargs}
+        surf = go.Isosurface(x=x, y=y, z=z, value=values.flatten(),
+                             **kwargs)
+        if show:
+            go.Figure(data=[surf]).show()
+        return surf
