@@ -61,21 +61,17 @@ def expm_ed_unit_inv(a_upp_r, oo_vo_blockonly=False):
     eigval[eigval.real < 1.0e-13] = 1.0e-13
     sqrt_eval = np.sqrt(eigval)
 
-    sin_sqrt_p = matrix_function(sqrt_eval, evec, np.sin)
     cos_sqrt_p = matrix_function(sqrt_eval, evec, np.cos)
-    sqrt_inv_p = matrix_function(1.0 / sqrt_eval, evec)
-
-    psin = sqrt_inv_p @ sin_sqrt_p
+    psin = matrix_function(sqrt_eval / np.pi, evec, np.sinc)
     u_oo = cos_sqrt_p
     u_vo = - a_upp_r.T.conj() @ psin
 
     if not oo_vo_blockonly:
-        inv_p = matrix_function(1.0 / eigval, evec)
         u_ov = psin @ a_upp_r
         dim_v = a_upp_r.shape[1]
-        dim_o = a_upp_r.shape[0]
-        u_vv = np.eye(dim_v) + \
-            a_upp_r.T.conj() @ (cos_sqrt_p - np.eye(dim_o)) @ inv_p @ a_upp_r
+
+        pcos = matrix_function((np.cos(sqrt_eval) - 1) / eigval, evec)
+        u_vv = np.eye(dim_v) + a_upp_r.T.conj() @ pcos @ a_upp_r
         u = np.vstack([
             np.hstack([u_oo, u_ov]),
             np.hstack([u_vo, u_vv])])
@@ -323,7 +319,7 @@ def sort_orbitals_according_to_occ_kpt(
 
         kpt.f_n = kpt.f_n[ind]
         if update_eps:
-            kpt.eps_n = kpt.eps_n[ind]
+            kpt.eps_n[:] = kpt.eps_n[ind]
 
         if update_mom:
             # OccupationsMOM.numbers needs
@@ -377,7 +373,9 @@ def sort_orbitals_according_to_energies(
             else:
                 sort_orbitals_kpt(wfs, kpt, ind, update_proj=True)
 
-            kpt.f_n[np.arange(len(ind))] = kpt.f_n[ind]
+            assert len(ind) == len(kpt.f_n)
+            # kpt.f_n[np.arange(len(ind))] = kpt.f_n[ind]
+            kpt.f_n = kpt.f_n[ind]
 
             if use_eps:
                 kpt.eps_n[np.arange(len(ind))] = orb_energies[ind]
