@@ -10,21 +10,24 @@ import numpy as np
 from ase import Atoms
 from ase.calculators.calculator import kpts2sizeandoffsets
 from ase.units import Bohr
-
 from gpaw.core import UGDesc
 from gpaw.core.atom_arrays import (AtomArrays, AtomArraysLayout,
                                    AtomDistribution)
 from gpaw.core.domain import Domain
 from gpaw.gpu.mpi import CuPyMPI
+from gpaw.lfc import BasisFunctions
 from gpaw.mixer import MixerWrapper, get_mixer_from_keywords
-from gpaw.mpi import (MPIComm, Parallelization, serial_comm, synchronize_atoms,
-                      world)
+from gpaw.mpi import (broadcast, MPIComm, Parallelization, serial_comm,
+                      synchronize_atoms, world)
 from gpaw.new import prod
 from gpaw.new.basis import create_basis
 from gpaw.new.brillouin import BZPoints, MonkhorstPackKPoints
+from gpaw.new.c import GPU_AWARE_MPI
 from gpaw.new.density import Density
 from gpaw.new.ibzwfs import IBZWaveFunctions
 from gpaw.new.input_parameters import InputParameters
+from gpaw.new.logger import Logger
+from gpaw.new.potential import Potential
 from gpaw.new.scf import SCFLoop
 from gpaw.new.smearing import OccupationNumberCalculator
 from gpaw.new.symmetry import create_symmetries_object
@@ -33,7 +36,6 @@ from gpaw.setup import Setups
 from gpaw.typing import Array2D, ArrayLike1D, ArrayLike2D, DTypeLike
 from gpaw.utilities.gpts import get_number_of_grid_points
 from gpaw.xc import XC
-from gpaw.new.c import GPU_AWARE_MPI
 
 
 def builder(atoms: Atoms,
@@ -82,36 +84,51 @@ class DFTComponentsBuilder:
         self.soc = params.soc
         self.nspins = self.ncomponents % 3
         self.spin_degeneracy = self.ncomponents % 2 + 1
-
         if isinstance(params.xc, (dict, str)):
             self._xc = XC(params.xc, collinear=(self.ncomponents < 4),
                           xp=self.xp)
         else:
             self._xc = params.xc
 
-        self.setups = Setups(atoms.numbers,
-                             params.setups,
-                             params.basis,
-                             self._xc.get_setup_name(),
-                             world=comm)
+        self._backwards_comatible = params.experimental.get(
+            'backwards_compatible', True)
 
+        self.setups = Setups(
+            atoms.numbers,
+            params.setups,
+            params.basis,
+            self._xc.get_setup_name(),
+            world=comm,
+            backwards_compatible=self._backwards_comatible)
         if params.hund:
             c = params.charge / len(atoms)
             for a, setup in enumerate(self.setups):
                 self.initial_magmom_av[a, 2] = setup.get_hunds_rule_moment(c)
 
-        symmetries = create_symmetries_object(atoms,
-                                              self.setups.id_a,
-                                              self.initial_magmom_av,
-                                              params.symmetry)
+        symmetries = create_symmetries_object(
+            atoms,
+            setup_ids=self.setups.id_a,
+            magmoms=self.initial_magmom_av,
+            **{k: v for k, v in params.symmetry.items()
+               if k != 'time_reversal'},
+            _backwards_compatible=self._backwards_comatible)
+
+        use_time_reversal = params.symmetry.get('time_reversal', True)
+
+        symmetries._old_symmetry.time_reversal = use_time_reversal  # legacy
+        self.setups.set_symmetry(symmetries._old_symmetry)  # legacy
+
         if self.ncomponents == 4:
-            assert (len(symmetries) == 1 and not
-                    symmetries.symmetry.time_reversal)
+            assert (len(symmetries) == 1 and not use_time_reversal)
 
         bz = create_kpts(params.kpts, atoms)
-        self.ibz = symmetries.reduce(bz, strict=False)
+        self.ibz = bz.reduce(
+            symmetries,
+            strict=False,
+            comm=comm,
+            use_time_reversal=use_time_reversal)
 
-        d = parallel.get('domain', None)
+        d = parallel.get('domain', 1 if self._xc.type == 'HYB' else None)
         k = parallel.get('kpt', None)
         b = parallel.get('band', None)
         self.communicators = create_communicators(comm, len(self.ibz),
@@ -142,9 +159,9 @@ class DFTComponentsBuilder:
 
         self.grid, self.fine_grid = self.create_uniform_grids()
 
-        self.fracpos_ac = self.atoms.get_scaled_positions()
-        self.fracpos_ac %= 1
-        self.fracpos_ac %= 1
+        self.relpos_ac = self.atoms.get_scaled_positions()
+        self.relpos_ac %= 1
+        self.relpos_ac %= 1
 
         self.xc = self.create_xc_functional()
 
@@ -157,7 +174,7 @@ class DFTComponentsBuilder:
     @cached_property
     def atomdist(self) -> AtomDistribution:
         return AtomDistribution(
-            self.grid.ranks_from_fractional_positions(self.fracpos_ac),
+            self.grid.ranks_from_fractional_positions(self.relpos_ac),
             self.grid.comm)
 
     def create_uniform_grids(self):
@@ -177,12 +194,24 @@ class DFTComponentsBuilder:
     def wf_desc(self) -> Domain:
         return self.create_wf_description()
 
+    @cached_property
+    def gpu(self) -> bool:
+        """Are we running on a GPU?."""
+        if self.params.parallel.get('gpu', False):
+            from gpaw.gpu import cupy_is_fake
+            if cupy_is_fake and not os.environ.get('GPAW_CPUPY'):
+                raise ValueError(
+                    'Please set GPAW_CPUPY=1 if you really want to do GPU '
+                    'calculations with GPAW''s fake cupy library '
+                    '(gpaw.gpu.cpupy)')
+            return True
+        return False
+
     @cached_property
     def xp(self) -> ModuleType:
         """Array module: Numpy or Cupy."""
-        if self.params.parallel['gpu']:
-            from gpaw.gpu import cupy, cupy_is_fake
-            assert not cupy_is_fake or os.environ.get('GPAW_CPUPY')
+        if self.gpu:
+            from gpaw.gpu import cupy
             return cupy
         return np
 
@@ -202,7 +231,7 @@ class DFTComponentsBuilder:
                             self.grid,
                             self.setups,
                             self.dtype,
-                            self.fracpos_ac,
+                            self.relpos_ac,
                             self.communicators['w'],
                             self.communicators['k'],
                             self.communicators['b'])
@@ -230,8 +259,16 @@ class DFTComponentsBuilder:
             self.communicators,
             self.initial_magmom_av.sum(0),
             self.ncomponents,
+            self.nelectrons,
             np.linalg.inv(self.atoms.cell.complete()).T)
 
+    def create_ibz_wave_functions(self,
+                                  basis: BasisFunctions,
+                                  potential: Potential,
+                                  *,
+                                  log: Logger) -> IBZWaveFunctions:
+        raise NotImplementedError
+
     def create_hamiltonian_operator(self):
         raise NotImplementedError
 
@@ -280,29 +317,41 @@ class DFTComponentsBuilder:
         occ_skn = reader.wave_functions.occupations
 
         for wfs in ibzwfs:
-            wfs._eig_n = eig_skn[wfs.spin, wfs.k] / ha
-            wfs._occ_n = occ_skn[wfs.spin, wfs.k]
+            index: tuple[int, ...]
+            if self.ncomponents < 4:
+                dims = [self.nbands]
+                index = (wfs.spin, wfs.k)
+            else:
+                dims = [self.nbands, 2]
+                index = (wfs.k,)
+
+            wfs._eig_n = eig_skn[index] / ha
+            wfs._occ_n = occ_skn[index]
             layout = AtomArraysLayout([(setup.ni,) for setup in self.setups],
                                       atomdist=self.atomdist,
                                       dtype=self.dtype)
-            if self.ncomponents < 4:
-                dims = [self.nbands]
-                index = [wfs.spin, wfs.k]
-            else:
-                dims = [self.nbands, 2]
-                index = [wfs.k]
-
             P_ani = AtomArrays(layout, dims=dims, comm=band_comm)
 
             if domain_comm.rank == 0:
-                P_nI = reader.wave_functions.proxy('projections', *index)
-                b1, b2 = P_ani.my_slice()  # my bands
-                data = P_nI[b1:b2].astype(ibzwfs.dtype)  # read from file
+                try:
+                    P_nI = reader.wave_functions.proxy('projections', *index)
+                except KeyError:
+                    data = None
+                else:
+                    b1, b2 = P_ani.my_slice()  # my bands
+                    data = P_nI[b1:b2].astype(ibzwfs.dtype)  # read from file
             else:
                 data = None
 
-            P_ani.scatter_from(data)  # distribute over atoms
-            wfs._P_ani = P_ani
+            have_projections = broadcast(
+                data is not None if domain_comm.rank == 0 else None,
+                comm=domain_comm)
+
+            if have_projections:
+                P_ani.scatter_from(data)  # distribute over atoms
+                wfs._P_ani = P_ani
+            else:
+                wfs._P_ani = None
 
         try:
             ibzwfs.fermi_levels = reader.wave_functions.fermi_levels / ha
@@ -390,13 +439,17 @@ def normalize_initial_magmoms(
 
 def create_kpts(kpts: dict[str, Any], atoms: Atoms) -> BZPoints:
     if 'kpts' in kpts:
-        assert len(kpts) == 1, kpts
-        return BZPoints(kpts['kpts'])
-    if 'path' in kpts:
+        bz = BZPoints(kpts['kpts'])
+    elif 'path' in kpts:
         path = atoms.cell.bandpath(pbc=atoms.pbc, **kpts)
-        return BZPoints(path.kpts)
-    size, offset = kpts2sizeandoffsets(**kpts, atoms=atoms)
-    return MonkhorstPackKPoints(size, offset)
+        bz = BZPoints(path.kpts)
+    else:
+        size, offset = kpts2sizeandoffsets(**kpts, atoms=atoms)
+        bz = MonkhorstPackKPoints(size, offset)
+    for c, periodic in enumerate(atoms.pbc):
+        if not periodic and not np.allclose(bz.kpt_Kc[:, c], 0.0):
+            raise ValueError('K-points can only be used with PBCs!')
+    return bz
 
 
 def calculate_number_of_bands(nbands: int | str | None,
@@ -453,7 +506,7 @@ def create_uniform_grid(mode: str,
                         gpts,
                         cell,
                         pbc,
-                        symmetry,
+                        symmetries,
                         h: float = None,
                         interpolation: str = None,
                         ecut: float = None,
@@ -474,5 +527,5 @@ def create_uniform_grid(mode: str,
     else:
         modeobj = SimpleNamespace(name=mode, ecut=ecut)
         size = get_number_of_grid_points(cell, h, modeobj, realspace,
-                                         symmetry.symmetry)
+                                         symmetries)
     return UGDesc(cell=cell, pbc=pbc, zerobc=zerobc, size=size, comm=comm)
