@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from functools import cached_property
-from typing import Generator, Generic, TypeVar, TYPE_CHECKING
+from typing import TYPE_CHECKING, Callable, Generator, Generic, TypeVar
 
 import numpy as np
 from ase.io.ulm import Writer
@@ -16,6 +16,7 @@ from gpaw.new.potential import Potential
 from gpaw.new.pwfd.wave_functions import PWFDWaveFunctions
 from gpaw.new.wave_functions import WaveFunctions
 from gpaw.typing import Array1D, Array2D, Self
+from gpaw.utilities import pack_density
 
 if TYPE_CHECKING:
     from gpaw.new.density import Density
@@ -66,6 +67,10 @@ class IBZWaveFunctions(Generic[WFT]):
             if not GPU_AWARE_MPI:
                 self.kpt_comm = CuPyMPI(self.kpt_comm)  # type: ignore
 
+        self.move_wave_functions: Callable[..., None] = lambda *args: None
+
+        self.read_from_file_init_wfs_dm = False
+
     @classmethod
     def create(cls,
                *,
@@ -110,6 +115,9 @@ class IBZWaveFunctions(Generic[WFT]):
             return 'fd'
         return 'lcao'
 
+    def has_wave_functions(self):
+        return True
+
     def get_max_shape(self, global_shape: bool = False) -> tuple[int, ...]:
         """Find the largest wave function array shape.
 
@@ -160,17 +168,18 @@ class IBZWaveFunctions(Generic[WFT]):
         for wfs_s in self.wfs_qs:
             yield from wfs_s
 
-    def move(self, fracpos_ac, atomdist):
-        self.ibz.symmetries.check_positions(fracpos_ac)
+    def move(self, relpos_ac, atomdist):
+        self.ibz.symmetries.check_positions(relpos_ac)
         self.energies.clear()
+        self.make_sure_wfs_are_read_from_gpw_file()
         for wfs in self:
-            wfs.move(fracpos_ac, atomdist)
+            wfs.move(relpos_ac, atomdist, self.move_wave_functions)
 
     def orthonormalize(self, work_array_nX: np.ndarray = None):
         for wfs in self:
             wfs.orthonormalize(work_array_nX)
 
-    def calculate_occs(self, occ_calc, fixed_fermi_level=False):
+    def calculate_occs(self, occ_calc, fix_fermi_level=False):
         degeneracy = self.spin_degeneracy
 
         # u index is q and s combined
@@ -180,9 +189,10 @@ class IBZWaveFunctions(Generic[WFT]):
             weights=[wfs.weight for wfs in self],
             fermi_levels_guess=(None
                                 if self.fermi_levels is None else
-                                self.fermi_levels * Ha))
+                                self.fermi_levels * Ha),
+            fix_fermi_level=fix_fermi_level)
 
-        if not fixed_fermi_level:
+        if not fix_fermi_level:
             self.fermi_levels = np.array(fermi_levels) / Ha
         else:
             assert self.fermi_levels is not None
@@ -243,7 +253,7 @@ class IBZWaveFunctions(Generic[WFT]):
 
         if not skip_paw_correction:
             dphi_aj = wfs.setups.partial_wave_corrections()
-            dphi_air = grid.atom_centered_functions(dphi_aj, wfs.fracpos_ac)
+            dphi_air = grid.atom_centered_functions(dphi_aj, wfs.relpos_ac)
             dphi_air.add_to(psi_r, wfs.P_ani[:, 0])
 
         return psi_r
@@ -290,8 +300,8 @@ class IBZWaveFunctions(Generic[WFT]):
             eig_skn = np.empty((self.nspins, nkpts, self.nbands))
             occ_skn = np.empty((self.nspins, nkpts, self.nbands))
         else:
-            eig_skn = np.empty((0, 0, 0))
-            occ_skn = np.empty((0, 0, 0))
+            eig_skn = np.empty((self.nspins, nkpts, 0))
+            occ_skn = np.empty((self.nspins, nkpts, 0))
         for k in range(nkpts):
             for s in range(self.nspins):
                 eig_n, occ_n = self.get_eigs_and_occs(k, s)
@@ -302,7 +312,7 @@ class IBZWaveFunctions(Generic[WFT]):
 
     def forces(self, potential: Potential) -> Array2D:
         self.make_sure_wfs_are_read_from_gpw_file()
-        F_av = self.xp.zeros((potential.dH_asii.natoms, 3))
+        F_av = self.xp.zeros((len(potential.dH_asii), 3))
         for wfs in self:
             wfs.force_contribution(potential, F_av)
         if self.xp is not np:
@@ -312,24 +322,29 @@ class IBZWaveFunctions(Generic[WFT]):
 
     def write(self,
               writer: Writer,
-              skip_wfs: bool) -> None:
+              skip_wfs: bool,
+              precision: str = 'double',
+              include_projections=True) -> None:
         """Write fermi-level(s), eigenvalues, occupation numbers, ...
 
         ... k-points, symmetry information, projections and possibly
         also the wave functions.
         """
         eig_skn, occ_skn = self.get_all_eigs_and_occs()
+        if not self.collinear:
+            eig_skn = eig_skn[0]
+            occ_skn = occ_skn[0]
         assert self.fermi_levels is not None
         writer.write(fermi_levels=self.fermi_levels * Ha,
                      eigenvalues=eig_skn * Ha,
                      occupations=occ_skn)
         ibz = self.ibz
         writer.child('kpts').write(
-            atommap=ibz.symmetries.a_sa,
+            atommap=ibz.symmetries.atommap_sa,
             bz2ibz=ibz.bz2ibz_K,
             bzkpts=ibz.bz.kpt_Kc,
             ibzkpts=ibz.kpt_kc,
-            rotations=ibz.symmetries.rotation_svv,
+            rotations=ibz.symmetries.rotation_scc,
             translations=ibz.symmetries.translation_sc,
             weights=ibz.weight_k)
 
@@ -345,37 +360,46 @@ class IBZWaveFunctions(Generic[WFT]):
             spin_k_shape = (len(ibz),)
             proj_shape = (self.nbands, 2, nproj)
 
-        writer.add_array('projections', spin_k_shape + proj_shape, self.dtype)
-
-        for spin in range(self.nspins):
-            for k, rank in enumerate(self.rank_k):
-                if rank == self.kpt_comm.rank:
-                    wfs = self.wfs_qs[self.q_k[k]][spin]
-                    P_ani = wfs.P_ani.to_cpu().gather()  # gather atoms
-                    if P_ani is not None:
-                        P_nI = P_ani.matrix.gather()  # gather bands
-                        if P_nI.dist.comm.rank == 0:
-                            if rank == 0:
-                                writer.fill(P_nI.data.reshape(proj_shape))
-                            else:
-                                self.kpt_comm.send(P_nI.data, 0)
-                elif self.comm.rank == 0:
-                    data = np.empty(proj_shape, self.dtype)
-                    self.kpt_comm.receive(data, rank)
-                    writer.fill(data)
+        if include_projections:
+            writer.add_array('projections', spin_k_shape + proj_shape,
+                             self.dtype)
+            for spin in range(self.nspins):
+                for k, rank in enumerate(self.rank_k):
+                    if rank == self.kpt_comm.rank:
+                        wfs = self.wfs_qs[self.q_k[k]][spin]
+                        P_ani = wfs.P_ani.to_cpu().gather()  # gather atoms
+                        if P_ani is not None:
+                            P_nI = P_ani.matrix.gather()  # gather bands
+                            if P_nI.dist.comm.rank == 0:
+                                if rank == 0:
+                                    writer.fill(P_nI.data.reshape(proj_shape))
+                                else:
+                                    self.kpt_comm.send(P_nI.data, 0)
+                    elif self.comm.rank == 0:
+                        data = np.empty(proj_shape, self.dtype)
+                        self.kpt_comm.receive(data, rank)
+                        writer.fill(data)
 
         if not skip_wfs:
-            self._write_wave_functions(writer, spin_k_shape)
+            self._write_wave_functions(writer, spin_k_shape, precision)
 
-    def _write_wave_functions(self, writer, spin_k_shape):
+    def _write_wave_functions(self, writer, spin_k_shape, precision='double'):
+        from gpaw.new.gpw import as_single_precision
         # We collect all bands to master.  This may have to be changed
         # to only one band at a time XXX
         xshape = self.get_max_shape(global_shape=True)
         shape = spin_k_shape + (self.nbands,) + xshape
         dtype = complex if self.mode == 'pw' else self.dtype
+        dtype_write = dtype
+        singlep = precision == 'single'
+        if singlep:
+            if dtype == complex:
+                dtype_write = np.complex64
+            else:
+                dtype_write = np.float32
         c = 1.0 if self.mode == 'lcao' else Bohr**-1.5
 
-        writer.add_array('coefficients', shape, dtype=dtype)
+        writer.add_array('coefficients', shape, dtype=dtype_write)
         buf_nX = np.empty((self.nbands,) + xshape, dtype=dtype)
 
         for spin in range(self.nspins):
@@ -395,12 +419,18 @@ class IBZWaveFunctions(Generic[WFT]):
                                 buf_nX[..., x:] = 0.0
                                 coef_nX = buf_nX
                         if rank == 0:
-                            writer.fill(coef_nX * c)
+                            if singlep:
+                                writer.fill(as_single_precision(coef_nX * c))
+                            else:
+                                writer.fill(coef_nX * c)
                         else:
                             self.kpt_comm.send(coef_nX, 0)
                 elif self.comm.rank == 0:
                     self.kpt_comm.receive(buf_nX, rank)
-                    writer.fill(buf_nX * c)
+                    if singlep:
+                        writer.fill(as_single_precision(buf_nX * c))
+                    else:
+                        writer.fill(buf_nX * c)
 
     def write_summary(self, log):
         fl = self.fermi_levels * Ha
@@ -474,30 +504,58 @@ class IBZWaveFunctions(Generic[WFT]):
             if psit_nX is None:
                 return
             if hasattr(psit_nX.data, 'fd'):  # fd=file-descriptor
+                self.read_from_file_init_wfs_dm = True
                 psit_nX.data = psit_nX.data[:]  # read
 
     def get_homo_lumo(self, spin: int = None) -> Array1D:
         """Return HOMO and LUMO eigenvalues."""
         if self.ncomponents == 1:
-            N = 2
             assert spin != 1
             spin = 0
         elif self.ncomponents == 2:
-            N = 2
             if spin is None:
                 h0, l0 = self.get_homo_lumo(0)
                 h1, l1 = self.get_homo_lumo(1)
                 return np.array([max(h0, h1), min(l0, l1)])
         else:
-            N = 1
             assert spin != 1
             spin = 0
 
-        n = int(round(self.nelectrons)) // N
-        assert N * n == self.nelectrons
-        homo = self.kpt_comm.max_scalar(max(wfs_s[spin].eig_n[n - 1]
-                                            for wfs_s in self.wfs_qs))
-        lumo = self.kpt_comm.min_scalar(min(wfs_s[spin].eig_n[n]
-                                            for wfs_s in self.wfs_qs))
+        nocc = 0.0
+        for wfs_s in self.wfs_qs:
+            wfs = wfs_s[spin]
+            nocc += wfs.occ_n.sum() * wfs.weight
+        nocc = self.kpt_comm.sum_scalar(nocc)
+        n = int(round(nocc))
+
+        homo = -np.inf
+        if n > 0:
+            for wfs_s in self.wfs_qs:
+                homo = max(homo, wfs_s[spin].eig_n[n - 1])
+        homo = self.kpt_comm.max_scalar(homo)
+
+        lumo = np.inf
+        if n < self.nbands:
+            for wfs_s in self.wfs_qs:
+                lumo = min(lumo, wfs_s[spin].eig_n[n])
+        lumo = self.kpt_comm.min_scalar(lumo)
 
         return np.array([homo, lumo])
+
+    def calculate_kinetic_energy(self,
+                                 hamiltonian,
+                                 density: Density) -> float:
+        e_kin = 0.0
+        for wfs in self:
+            e_kin += hamiltonian.calculate_kinetic_energy(wfs, skip_sum=True)
+        e_kin = self.comm.sum_scalar(e_kin)
+
+        # PAW corrections:
+        e_kin_paw = 0.0
+        for a, D_sii in density.D_asii.items():
+            setup = wfs.setups[a]
+            D_p = pack_density(D_sii.real[:density.ndensities].sum(0))
+            e_kin_paw += setup.K_p @ D_p + setup.Kc
+        e_kin_paw = density.grid.comm.sum_scalar(e_kin_paw)
+
+        return e_kin + e_kin_paw
