@@ -12,7 +12,8 @@ class PWDescriptor:
     ndim = 1  # all 3d G-vectors are stored in a 1d ndarray
 
     def __init__(self, ecut, gd, dtype=None, kd=None,
-                 fftwflags=fftw.MEASURE, gammacentered=False):
+                 fftwflags=fftw.MEASURE, gammacentered=False,
+                 _new=False):
 
         assert gd.pbc_c.all()
 
@@ -110,7 +111,13 @@ class PWDescriptor:
         self.G2_qG = []
         self.myQ_qG = []
         self.myng_q = []
+        self.maxmyng_q = []
         for q, G2_G in enumerate(G2_qG):
+            if _new:
+                x = (self.ng_q[q] + S - 1) // S
+                self.maxmyng_q.append(x)
+                ng1 = gd.comm.rank * x
+                ng2 = ng1 + x
             G2_G = G2_G[ng1:ng2].copy()
             G2_G.flags.writeable = False
             self.G2_qG.append(G2_G)
@@ -123,6 +130,8 @@ class PWDescriptor:
         else:
             self.tmp_G = None
 
+        self._new = _new
+
     def get_reciprocal_vectors(self, q=0, add_q=True):
         """Returns reciprocal lattice vectors plus q, G + q,
         in xyz coordinates."""
@@ -236,7 +245,6 @@ class PWDescriptor:
             Q_G = self.Q_qG[q]
             assert len(c_G) == len(Q_G)
             cgpaw.pw_insert(c_G, Q_G, scale, self.tmp_Q)
-
             if self.dtype == float:
                 t = self.tmp_Q[:, :, 0]
                 n, m = self.gd.N_c[:2] // 2 - 1
@@ -292,7 +300,8 @@ class PWDescriptor:
         ssize_r[:N] = self.myng_q[q]
         soffset_r = np.arange(comm.size) * self.myng_q[q]
         soffset_r[N:] = 0
-        roffset_r = (np.arange(comm.size) * self.maxmyng).clip(max=ng)
+        myng = self.maxmyng_q[q] if self._new else self.maxmyng
+        roffset_r = (np.arange(comm.size) * myng).clip(max=ng)
         rsize_r = np.zeros(comm.size, int)
         if comm.rank < N:
             rsize_r[:-1] = roffset_r[1:] - roffset_r[:-1]
@@ -314,7 +323,8 @@ class PWDescriptor:
         rsize_r[:N] = self.myng_q[q]
         roffset_r = np.arange(comm.size) * self.myng_q[q]
         roffset_r[N:] = 0
-        soffset_r = (np.arange(comm.size) * self.maxmyng).clip(max=ng)
+        myng = self.maxmyng_q[q] if self._new else self.maxmyng
+        soffset_r = (np.arange(comm.size) * myng).clip(max=ng)
         ssize_r = np.zeros(comm.size, int)
         if comm.rank < N:
             ssize_r[:-1] = soffset_r[1:] - soffset_r[:-1]
