@@ -11,7 +11,7 @@ import gpaw.mpi as mpi
 from gpaw.response.pw_parallelization import Blocks1D
 from gpaw.response.coulomb_kernels import CoulombKernel
 from gpaw.response.dyson import DysonEquation
-from gpaw.response.density_kernels import get_density_xc_kernel
+from gpaw.response.density_kernels import DensityXCKernel
 from gpaw.response.chi0 import Chi0Calculator, get_frequency_descriptor
 from gpaw.response.chi0_data import Chi0Data
 from gpaw.response.pair import get_gs_and_context
@@ -46,12 +46,11 @@ V or V(q): The modified Coulomb interaction. Equal to V(q) for finite
 @dataclass
 class Chi0DysonEquations:
     chi0: Chi0Data
-    df: 'DielectricFunctionCalculator'
+    coulomb: CoulombKernel
+    xc_kernel: DensityXCKernel | None
+    cd: CellDescriptor
 
     def __post_init__(self):
-        self.gs = self.df.gs
-        self.context = self.df.context
-        self.coulomb = self.df.coulomb
         if self.coulomb.truncation is None:
             self.bare_coulomb = self.coulomb
         else:
@@ -83,16 +82,7 @@ class Chi0DysonEquations:
             chi0_wGG[:, 0, 0] = np.dot(d_v, np.dot(chi0.chi0_Wvv[W_w], d_v).T)
         return chi0_wGG
 
-    def get_Kxc_GG(self, *, xc, chi0_wGG, **kwargs):
-        """Get adiabatic xc kernel (TDDFT).
-
-        Choose between ALDA, Bootstrap and LRalpha (long-range kernel), where
-        alpha is a user specified parameter (for example xc='LR0.25')."""
-        return get_density_xc_kernel(
-            self.chi0.qpd, self.gs, self.context,
-            functional=xc, chi0_wGG=chi0_wGG, **kwargs)
-
-    def get_coulomb_scaled_kernel(self, xc='RPA', modified=False, **xckwargs):
+    def get_coulomb_scaled_kernel(self, modified=False, Kxc_GG=None):
         """Get the Hxc kernel rescaled by the bare Coulomb potential v(q).
 
         Calculates
@@ -118,8 +108,7 @@ class Chi0DysonEquations:
             K_GG = np.diag(V_G / v_G)
         if modified:
             K_GG[0, 0] = 0.
-        if xc != 'RPA':
-            Kxc_GG = self.get_Kxc_GG(xc=xc, **xckwargs)
+        if Kxc_GG is not None:
             sqrtv_G = v_G**0.5
             K_GG += Kxc_GG / sqrtv_G / sqrtv_G[:, np.newaxis]
         return v_G, K_GG
@@ -143,7 +132,11 @@ class Chi0DysonEquations:
         return out_wGG
 
     def rpa_density_response(self, direction='x', qinf_v=None):
-        """Calculate the RPA susceptibility for (semi-)finite q."""
+        """Calculate the RPA susceptibility for (semi-)finite q.
+
+        Currently this is only used by the QEH code, why we don't support a top
+        level user interface.
+        """
         # Extract χ₀(q,ω)
         qpd = self.chi0.qpd
         chi0_wGG = self.get_chi0_wGG(direction=direction)
@@ -159,12 +152,12 @@ class Chi0DysonEquations:
         chi_wGG = self.invert_dyson_like_equation(chi0_wGG, V_GG)
         return qpd, chi_wGG, self.wblocks
 
-    def inverse_dielectric_function(self, *args, **kwargs):
+    def inverse_dielectric_function(self, direction='x'):
         """Calculate v^(1/2) χ v^(1/2), from which ε⁻¹(q,ω) is constructed."""
         return InverseDielectricFunction.from_chi0_dyson_eqs(
-            self, *self.calculate_vchi_symm(*args, **kwargs))
+            self, *self.calculate_vchi_symm(direction=direction))
 
-    def calculate_vchi_symm(self, xc='RPA', direction='x', **kwargs):
+    def calculate_vchi_symm(self, direction='x', modified=False):
         """Calculate v^(1/2) χ v^(1/2).
 
         Starting from the TDDFT Dyson equation
@@ -192,8 +185,10 @@ class Chi0DysonEquations:
         perturbation theory.
         """
         chi0_wGG = self.get_chi0_wGG(direction=direction)
+        Kxc_GG = self.xc_kernel(self.chi0.qpd, chi0_wGG=chi0_wGG) \
+            if self.xc_kernel else None
         v_G, K_GG = self.get_coulomb_scaled_kernel(
-            xc=xc, chi0_wGG=chi0_wGG, **kwargs)
+            modified=modified, Kxc_GG=Kxc_GG)
         # Calculate v^(1/2)(q) χ₀(q,ω) v^(1/2)(q)
         sqrtv_G = v_G**0.5
         vchi0_symm_wGG = chi0_wGG  # reuse buffer
@@ -204,10 +199,10 @@ class Chi0DysonEquations:
             vchi0_symm_wGG, K_GG, reuse_buffer=False)
         return vchi0_symm_wGG, vchi_symm_wGG
 
-    def customized_dielectric_function(self, *args, **kwargs):
+    def customized_dielectric_function(self, direction='x'):
         """Calculate Ε(q,ω) = 1 - V(q) P(q,ω)."""
         V_GG = self.coulomb.kernel(self.chi0.qpd)
-        P_wGG = self.polarizability_operator(*args, **kwargs)
+        P_wGG = self.polarizability_operator(direction=direction)
         nG = len(V_GG)
         eps_wGG = P_wGG  # reuse buffer
         for w, P_GG in enumerate(P_wGG):
@@ -215,7 +210,7 @@ class Chi0DysonEquations:
         return CustomizableDielectricFunction.from_chi0_dyson_eqs(
             self, eps_wGG)
 
-    def bare_dielectric_function(self, xc='RPA', direction='x', **ignored):
+    def bare_dielectric_function(self, direction='x'):
         """Calculate v^(1/2) ̄χ v^(1/2), from which ̄ϵ=1-v ̄χ is constructed.
 
         The unscreened susceptibility is given by the Dyson-like equation
@@ -231,17 +226,17 @@ class Chi0DysonEquations:
                                                                   ˍ
         We may thus reuse that functionality to calculate v^(1/2) χ v^(1/2).
         """
-        if xc != 'RPA':
+        if self.xc_kernel:
             raise NotImplementedError(
                 'Calculation of the bare dielectric function within TDDFT has '
                 'not yet been implemented. For TDDFT dielectric properties, '
                 'please calculate the inverse dielectric function.')
         vP_symm_wGG, vchibar_symm_wGG = self.calculate_vchi_symm(
-            xc=xc, direction=direction, modified=True)
+            direction=direction, modified=True)
         return BareDielectricFunction.from_chi0_dyson_eqs(
             self, vP_symm_wGG, vchibar_symm_wGG)
 
-    def polarizability_operator(self, xc='RPA', direction='x', **xckwargs):
+    def polarizability_operator(self, direction='x'):
         """Calculate the polarizability operator P(q,ω).
 
         Depending on the theory (RPA, TDDFT, MBPT etc.), the polarizability
@@ -255,7 +250,7 @@ class Chi0DysonEquations:
             P(q,ω) = [1 - χ₀(q,ω) K_xc(q,ω)]⁻¹ χ₀(q,ω)
         """
         chi0_wGG = self.get_chi0_wGG(direction=direction)
-        if xc == 'RPA':
+        if not self.xc_kernel:  # RPA
             return chi0_wGG
         # TDDFT (in adiabatic approximations to the kernel)
         if self.chi0.qpd.optical_limit:
@@ -275,7 +270,7 @@ class Chi0DysonEquations:
             # Dyson equation for χ, but unless we need to calculate the TDDFT
             # polarizability using truncated kernels, this isn't really
             # necessary.
-        Kxc_GG = self.get_Kxc_GG(xc=xc, chi0_wGG=chi0_wGG, **xckwargs)
+        Kxc_GG = self.xc_kernel(self.chi0.qpd, chi0_wGG=chi0_wGG)
         return self.invert_dyson_like_equation(chi0_wGG, Kxc_GG)
 
 
@@ -291,7 +286,7 @@ class DielectricFunctionData(ABC):
     @classmethod
     def from_chi0_dyson_eqs(cls, chi0_dyson_eqs, *args, **kwargs):
         chi0 = chi0_dyson_eqs.chi0
-        return cls(chi0_dyson_eqs.gs.cd, chi0.qpd,
+        return cls(chi0_dyson_eqs.cd, chi0.qpd,
                    chi0.wd, chi0_dyson_eqs.wblocks,
                    chi0_dyson_eqs.coulomb, chi0_dyson_eqs.bare_coulomb,
                    *args, **kwargs)
@@ -566,104 +561,94 @@ class BareDielectricFunction(DielectricFunctionData):
 
 
 class DielectricFunctionCalculator:
-    def __init__(self, chi0calc: Chi0Calculator, coulomb: CoulombKernel):
+    def __init__(self, chi0calc: Chi0Calculator):
         self.chi0calc = chi0calc
-        self.coulomb = coulomb
-
         self.gs = chi0calc.gs
         self.context = chi0calc.context
 
-        self._chi0cache: dict = {}
+        self._chi0cache: dict[tuple[str, ...], Chi0Data] = {}
 
-    def calculate_chi0(self, q_c: list | np.ndarray) -> Chi0DysonEquations:
-        """Calculates the response function.
+    def get_chi0(self, q_c: list | np.ndarray) -> Chi0Data:
+        """Get the Kohn-Sham susceptibility χ₀(q,ω) for input wave vector q.
 
-        Calculate the response function for a specific momentum.
-
-        q_c: [float, float, float]
-            The momentum wavevector.
+        Keeps a cache of χ₀ for the latest calculated wave vector, thus
+        allowing for investigation of multiple dielectric properties,
+        Coulomb truncations, xc kernels etc. without recalculating χ₀.
         """
-
-        # We cache the computed data since chi0 may otherwise be redundantly
-        # calculated e.g. if the user calculates multiple directions.
-        #
-        # May be called multiple times with same q_c, and we want to
-        # be able to recognize previous seen values of q_c.
-        # We do this by rounding and converting to string with fixed
-        # precision (so not very elegant).
+        # As cache key, we round off and use a string representation.
+        # Not very elegant, but it should work almost always.
         q_key = [f'{q:.10f}' for q in q_c]
         key = tuple(q_key)
-
         if key not in self._chi0cache:
-            # We assume that the caller will trigger this multiple
-            # times with the same qpoint, then several times with
-            # another qpoint, etc.  If that's true, then we
-            # need to cache no more than one qpoint at a time.
-            # Thus to save memory, we clear the cache here.
-            #
-            # This should be replaced with something more reliable,
-            # such as having the caller manage things more explicitly.
-            #
-            # See https://gitlab.com/gpaw/gpaw/-/issues/662
-            #
-            # In conclusion, delete the cache now:
             self._chi0cache.clear()
-
-            # cache Chi0Data from gpaw.response.chi0_data
-            self._chi0cache[key] = Chi0DysonEquations(
-                self.chi0calc.calculate(q_c), self)
+            self._chi0cache[key] = self.chi0calc.calculate(q_c)
             self.context.write_timer()
-
         return self._chi0cache[key]
 
-    def _dielectric_function_new(self, q_c=[0, 0, 0], direction='x',
-                                 **xckwargs) -> DielectricFunctionData:
-        # Temporary method until truncation becomes a method input XXX
-        chi0_dyson_eqs = self.calculate_chi0(q_c)
-        if self.coulomb.truncation:
-            # eps: BareDielectricFunction
-            method = chi0_dyson_eqs.bare_dielectric_function
+    def get_chi0_dyson_eqs(self,
+                           q_c: list | np.ndarray = [0, 0, 0],
+                           truncation: str | None = None,
+                           xc: str = 'RPA',
+                           **xckwargs
+                           ) -> Chi0DysonEquations:
+        """Set up the Dyson equation for χ(q,ω) at given wave vector q.
+
+        Parameters
+        ----------
+        truncation : str or None
+            Truncation of the Hartree kernel.
+        xc : str
+            Exchange-correlation kernel for LR-TDDFT calculations.
+            If xc == 'RPA', the dielectric response is treated in the random
+            phase approximation.
+        **xckwargs
+            Additional parameters for the chosen xc kernel.
+        """
+        chi0 = self.get_chi0(q_c)
+        coulomb = CoulombKernel.from_gs(self.gs, truncation=truncation)
+        if xc == 'RPA':
+            xc_kernel = None
         else:
-            # eps: CustomizableDielectricFunction
-            method = chi0_dyson_eqs.customized_dielectric_function
-        eps = method(direction=direction, **xckwargs)
-        return eps
+            xc_kernel = DensityXCKernel.from_functional(
+                self.gs, self.context, functional=xc, **xckwargs)
+        return Chi0DysonEquations(chi0, coulomb, xc_kernel, self.gs.cd)
 
-    def get_bare_dielectric_function(self, q_c=[0, 0, 0], direction='x',
-                                     **xckwargs) -> BareDielectricFunction:
-        return self.calculate_chi0(q_c).bare_dielectric_function(
-            direction=direction, **xckwargs)
+    def get_bare_dielectric_function(self, direction='x', **kwargs
+                                     ) -> BareDielectricFunction:
+        """Calculate the bare dielectric function ̄ϵ(q,ω) = 1 - v(q) ̄χ(q,ω).
 
-    def get_literal_dielectric_function(
-            self, *args, **kwargs) -> CustomizableDielectricFunction:
+        Here v(q) is the bare Coulomb potential while ̄χ is the unscreened
+        susceptibility calculated based on the modified (and possibly
+        truncated) Coulomb potential.
+        """
+        return self.get_chi0_dyson_eqs(
+            **kwargs).bare_dielectric_function(direction=direction)
+
+    def get_literal_dielectric_function(self, direction='x', **kwargs
+                                        ) -> CustomizableDielectricFunction:
         """Calculate the dielectric function ε(q,ω) = 1 - v(q) P(q,ω)."""
-        return self.get_customized_dielectric_function(
-            truncation=None, *args, **kwargs)
+        return self.get_chi0_dyson_eqs(
+            truncation=None, **kwargs).customized_dielectric_function(
+                direction=direction)
 
-    def get_customized_dielectric_function(
-            self, *, truncation: str | None,
-            q_c=[0, 0, 0], direction='x',
-            **xckwargs) -> CustomizableDielectricFunction:
-        # NB: ignores self.coulomb while this still exists XXX
-        chi0_dyson_equation = self.calculate_chi0(q_c)
-        if truncation is None:
-            chi0_dyson_equation.coulomb = chi0_dyson_equation.bare_coulomb
-        else:
-            chi0_dyson_equation.coulomb = chi0_dyson_equation.coulomb.new(
-                truncation=truncation)
-        return chi0_dyson_equation.customized_dielectric_function(
-            direction=direction, **xckwargs)
+    def get_customized_dielectric_function(self, direction='x', *,
+                                           truncation: str, **kwargs
+                                           ) -> CustomizableDielectricFunction:
+        """Calculate the customized dielectric function Ε(q,ω) = 1 -V(q)P(q,ω).
 
-    def get_inverse_dielectric_function(
-            self, q_c=[0, 0, 0], direction='x',
-            **xckwargs) -> InverseDielectricFunction:
-        return self.calculate_chi0(q_c).inverse_dielectric_function(
-            direction=direction, **xckwargs)
+        In comparison with the literal dielectric function, the bare Coulomb
+        interaction has here been replaced with a truncated analogue v(q)→V(q).
+        """
+        return self.get_chi0_dyson_eqs(
+            truncation=truncation, **kwargs).customized_dielectric_function(
+                direction=direction)
 
-    def get_rpa_density_response(self, q_c, *, direction, qinf_v=None):
-        # Used by the QEH code
-        return self.calculate_chi0(q_c).rpa_density_response(
-            direction=direction, qinf_v=qinf_v)
+    def get_inverse_dielectric_function(self, direction='x', **kwargs
+                                        ) -> InverseDielectricFunction:
+        """Calculate the inverse dielectric function ε⁻¹(q,ω) = v(q) χ(q,ω).
+        """
+        return self.get_chi0_dyson_eqs(
+            **kwargs).inverse_dielectric_function(direction=direction)
 
 
 class DielectricFunction(DielectricFunctionCalculator):
@@ -675,8 +660,8 @@ class DielectricFunction(DielectricFunctionCalculator):
                  hilbert=True,
                  nbands=None, eta=0.2,
                  intraband=True, nblocks=1, world=mpi.world, txt=sys.stdout,
-                 truncation=None, disable_point_group=False,
-                 disable_time_reversal=False,
+                 truncation=None,
+                 qsymmetry=True,
                  integrationmode=None, rate=0.0,
                  eshift: float | None = None):
         """Creates a DielectricFunction object.
@@ -722,14 +707,12 @@ class DielectricFunction(DielectricFunctionCalculator):
             ecut=ecut, nbands=nbands, eta=eta,
             hilbert=hilbert,
             intraband=intraband,
-            disable_point_group=disable_point_group,
-            disable_time_reversal=disable_time_reversal,
+            qsymmetry=qsymmetry,
             integrationmode=integrationmode,
             rate=rate, eshift=eshift
         )
-        coulomb = CoulombKernel.from_gs(gs, truncation=truncation)
-
-        super().__init__(chi0calc, coulomb)
+        super().__init__(chi0calc)
+        self.truncation = truncation
 
     def get_frequencies(self) -> np.ndarray:
         """Return frequencies (in eV) that the χ is evaluated on."""
@@ -739,7 +722,8 @@ class DielectricFunction(DielectricFunctionCalculator):
                                    filename='chiM_w.csv',
                                    **kwargs):
         dynsus = self.get_inverse_dielectric_function(
-            *args, xc=xc, **kwargs).dynamic_susceptibility()
+            *args, xc=xc, truncation=self.truncation,
+            **kwargs).dynamic_susceptibility()
         if filename:
             dynsus.write(filename)
         return dynsus.unpack()
@@ -757,7 +741,8 @@ class DielectricFunction(DielectricFunctionCalculator):
             Dielectric functio with local field corrections.
         """
         df = self.get_inverse_dielectric_function(
-            *args, **kwargs).macroscopic_dielectric_function()
+            *args, truncation=self.truncation,
+            **kwargs).macroscopic_dielectric_function()
         if filename:
             df.write(filename)
         return df.unpack()
@@ -775,13 +760,14 @@ class DielectricFunction(DielectricFunctionCalculator):
             Fully screened EELS spectrum.
         """
         eels = self.get_inverse_dielectric_function(
-            *args, **kwargs).eels_spectrum()
+            *args, truncation=self.truncation, **kwargs).eels_spectrum()
         if filename:
             eels.write(filename)
         return eels.unpack()
 
-    def get_polarizability(self, *args, filename='polarizability.csv',
-                           **kwargs):
+    def get_polarizability(self, q_c: list | np.ndarray = [0, 0, 0],
+                           direction='x', filename='polarizability.csv',
+                           **xckwargs):
         """Calculate the macroscopic polarizability.
 
         Generate a file 'polarizability.csv', unless filename is set to None.
@@ -793,8 +779,16 @@ class DielectricFunction(DielectricFunctionCalculator):
         alpha_w: np.ndarray
             Polarizability calculated with local-field corrections.
         """
-        pol = self._dielectric_function_new(
-            *args, **kwargs).polarizability()
+        chi0_dyson_eqs = self.get_chi0_dyson_eqs(
+            q_c, truncation=self.truncation, **xckwargs)
+        if self.truncation:
+            # eps: BareDielectricFunction
+            method = chi0_dyson_eqs.bare_dielectric_function
+        else:
+            # eps: CustomizableDielectricFunction
+            method = chi0_dyson_eqs.customized_dielectric_function
+        eps = method(direction=direction)
+        pol = eps.polarizability()
         if filename:
             pol.write(filename)
         return pol.unpack()
