@@ -23,10 +23,19 @@ import numpy as np
 
 from math import pi
 from collections import defaultdict
-from gpaw.cgpaw import spherical_harmonics as Yl
+from gpaw.cgpaw import spherical_harmonics
+from gpaw import debug
 
 __all__ = ['Y', 'YL', 'nablarlYL', 'Yl']
 
+if debug:
+    def Yl(l: int, R_v: np.ndarray, rlY_m: np.ndarray) -> None:
+        assert R_v.dtype == float and R_v.shape == (3,)
+        assert rlY_m.dtype == float and rlY_m.shape == (2 * l + 1,)
+        spherical_harmonics(l, R_v, rlY_m)
+else:
+    Yl = spherical_harmonics
+
 names = [['1'],
          ['y', 'z', 'x'],
          ['xy', 'yz', '3z2-r2', 'zx', 'x2-y2'],
