@@ -5,7 +5,7 @@ from gpaw import GPAW
 from gpaw.test import gen
 
 
-@pytest.mark.later
+@pytest.mark.old_gpaw_only
 def test_S2p():
     """Compare number of electrons for different channels with 2p corehole"""
     atoms = molecule('SH2')
