@@ -746,6 +746,19 @@ class GPWFiles(CachedFilesHandler):
         atoms.calc.diagonalize_full_hamiltonian(nbands=8, scalapack=scalapack)
         return atoms.calc
 
+    @gpwfile
+    def c2_gw_more_bands(self):
+        a = 3.567
+        atoms = bulk('C', 'diamond', a=a)
+        atoms.calc = GPAW(mode=PW(400),
+                          parallel={'domain': 1},
+                          kpts={'size': (2, 2, 2), 'gamma': True},
+                          xc='LDA',
+                          occupations=FermiDirac(0.001))
+        atoms.get_potential_energy()
+        atoms.calc.diagonalize_full_hamiltonian(nbands=128)
+        return atoms.calc
+
     @gpwfile
     def na_pw(self):
         from ase.build import bulk
@@ -844,7 +857,7 @@ class GPWFiles(CachedFilesHandler):
     def na2_isolated(self):
         # Permittivity file
         if world.rank == 0:
-            fo = open('ed.txt', 'w')
+            fo = open(self.folder / 'ed.txt', 'w')
             fo.writelines(['1.20 0.20 25.0'])
             fo.close()
         world.barrier()
@@ -869,7 +882,7 @@ class GPWFiles(CachedFilesHandler):
         sphere_center = np.array([10.0, 10.0, 10.0])
         classical_material.add_component(
             PolarizableSphere(
-                permittivity=PermittivityPlus('ed.txt'),
+                permittivity=PermittivityPlus(self.folder / 'ed.txt'),
                 center=sphere_center,
                 radius=5.0
             )
@@ -998,7 +1011,7 @@ class GPWFiles(CachedFilesHandler):
                     spinpol=spinpol,
                     communicator=world,
                     symmetry={'point_group': False},
-                    txt='gs.out')
+                    txt=self.folder / 'gs.out')
         atoms.calc = calc
         atoms.get_potential_energy()
         return atoms.calc
@@ -1754,3 +1767,10 @@ class GPWFiles(CachedFilesHandler):
 # We add Si fixtures with various symmetries to the GPWFiles namespace
 for name, method in si_gpwfiles().items():
     setattr(GPWFiles, name, method)
+
+
+if __name__ == '__main__':
+    import sys
+    name = sys.argv[1]
+    calc = getattr(GPWFiles(Path()), name)()
+    calc.write(name + '.gpw', mode='all')
