@@ -7,10 +7,7 @@ from gpaw.mpi import world
 
 
 @pytest.mark.response
-def test_gw_spinpol(in_tmp_dir, gpw_files, gpaw_new):
-    if gpaw_new and world.size > 1:
-        pytest.skip('Hybrids not working in parallel with GPAW_NEW=1')
-
+def test_gw_spinpol(in_tmp_dir, gpw_files):
     if world.size > 1:
         nblocks = 2
     else:
