@@ -7,7 +7,7 @@ from gpaw import GPAW, FermiDirac
 from gpaw.analyse.hirshfeld import HirshfeldPartitioning
 
 
-@pytest.mark.later
+@pytest.mark.old_gpaw_only
 def test_vdw_H_Hirshfeld():
     h = 0.25
     box = 3
