@@ -203,7 +203,7 @@ class WaveFunctions:
             D_asp, a_sa=self.kd.symmetry.a_sa)
         D_asp.redistribute(self.atom_partition)
 
-    def calculate_occupation_numbers(self, fixed_fermi_level=False):
+    def calculate_occupation_numbers(self, fix_fermi_level=False):
         if self.collinear and self.nspins == 1:
             degeneracy = 2
         else:
@@ -213,10 +213,11 @@ class WaveFunctions:
             nelectrons=self.nvalence / degeneracy,
             eigenvalues=[kpt.eps_n * Ha for kpt in self.kpt_u],
             weights=[kpt.weightk for kpt in self.kpt_u],
-            fermi_levels_guess=self.fermi_levels * Ha
-            if self.fermi_levels is not None else None)
+            fermi_levels_guess=(self.fermi_levels * Ha
+                                if self.fermi_levels is not None else None),
+            fix_fermi_level=fix_fermi_level)
 
-        if not fixed_fermi_level or self.fermi_levels is None:
+        if not fix_fermi_level or self.fermi_levels is None:
             self.fermi_levels = np.array(fermi_levels) / Ha
 
         for f_n, kpt in zip(f_qn, self.kpt_u):
@@ -429,7 +430,7 @@ class WaveFunctions:
         # and there.
         return np.nan
 
-    def get_homo_lumo(self, spin=None):
+    def get_homo_lumo(self, spin=None, _gllb=False):
         """Return HOMO and LUMO eigenvalues."""
         if spin is None:
             if self.nspins == 1:
@@ -438,7 +439,17 @@ class WaveFunctions:
             h1, l1 = self.get_homo_lumo(1)
             return np.array([max(h0, h1), min(l0, l1)])
 
-        n = self.nvalence // 2
+        if _gllb:
+            # Backwards compatibility (see test/gllb/test_metallic.py test)
+            n = self.nvalence // 2
+        else:
+            nocc = 0.0
+            for kpt in self.kpt_u:
+                if kpt.s == spin:
+                    nocc += kpt.f_n.sum()
+            nocc = self.kptband_comm.sum_scalar(nocc) * self.nspins / 2
+            n = int(round(nocc))
+
         band_rank, myn = self.bd.who_has(n - 1)
         homo = -np.inf
         if self.bd.comm.rank == band_rank:
